from copy import deepcopy

from wagtail.wagtailcore import blocks

from streamfield_tools.blocks import (
    MultiRenditionStructBlock,
    Rendition,
    RenditionAwareLazyLoadImageChooserBlock,
    RenditionAwareListBlock,
    RenditionAwareStructBlock
)

stacks_image_form_blocks = [
    (
        'image',
        RenditionAwareLazyLoadImageChooserBlock(
            icon='image',
            abel='Image'
        )
    ),
    ('overline', blocks.TextBlock(required=False)),
    ('title', blocks.TextBlock(required=False)),
    ('additional_content', blocks.RichTextBlock(required=False))
]


stacks_image_block = MultiRenditionStructBlock(
    deepcopy(stacks_image_form_blocks),
    core_renditions=(
        Rendition(
            short_name='full_width',
            verbose_name="Full Width Image",
            description="An image that spans the full width of it's "
                        "containing div.",
            path_to_template='wagtail_stacks_image/single/'
                             'full_width.html',
            image_rendition='fill-1600x901'
        ),
        Rendition(
            short_name='image_left_content_right',
            verbose_name="Image Left, Content Right",
            description="Display an image on the left with it's accompanying "
                        "content on the right. NOTE: This rendition should "
                        "only be used if the 'Optional Content' field is "
                        "filled out.",
            path_to_template='wagtail_stacks_image/single/'
                             'image_left_content_right.html',
            image_rendition='fill-800x450'
        ),
        Rendition(
            short_name='image_right_content_left',
            verbose_name="Image Right, Content Left",
            description="Display an image on the right with it's accompanying "
                        "content on the left. NOTE: This rendition should "
                        "only be used if the 'Optional Content' field is "
                        "filled out.",
            path_to_template='wagtail_stacks_image/single/'
                             'image_right_content_left.html',
            image_rendition='fill-800x450'
        )
    ),
    addl_renditions_settings_key='stacks_image_block',
    label='Image',
    icon='image'
)


stacks_imagelist_block = MultiRenditionStructBlock(
    [
        ('title', blocks.TextBlock(required=False)),
        (
            'list', RenditionAwareListBlock(
                RenditionAwareStructBlock(
                    deepcopy(stacks_image_form_blocks),
                    template='wagtail_stacks_image/list/'
                             'list-item.html',
                    template_carousel='wagtail_stacks_image/list/'
                                      'list-item-carousel.html'
                ),
                label='Images'
            ),
        )
    ],
    [
        Rendition(
            short_name='1up',
            verbose_name="Image List 1-Up",
            description="A list of images displayed in grid with one image "
                        "in each row.",
            path_to_template='wagtail_stacks_image/list/1up.html',
            image_rendition='fill-1600x901'
        ),
        Rendition(
            short_name='2up',
            verbose_name="Image List 2-Up",
            description="A list of images displayed in grid with two "
                        "in each row.",
            path_to_template='wagtail_stacks_image/list/2up.html',
            image_rendition='fill-800x450'
        ),
        Rendition(
            short_name='3up',
            verbose_name="Image List 3-Up",
            description="A list of images displayed in grid with three "
                        "in each row.",
            path_to_template='wagtail_stacks_image/list/3up.html',
            image_rendition='fill-700x394'
        ),
        Rendition(
            short_name='carousel',
            verbose_name="Image Carousel",
            description="A list of images displayed in a javascript-powered "
                        "carousel.",
            path_to_template='wagtail_stacks_image/list/carousel.html',
            image_rendition='max-1600x901'
        )
    ],
    addl_renditions_settings_key='stacks_imagelist_block',
    label='Image List/Gallery',
    icon='image'
)



from __future__ import unicode_literals
import json

from django.contrib.auth.models import User
from django.test import Client, TestCase

from stacks_image.models import StacksImageList


class StacksImageTestCase(TestCase):
    """The test suite for stacks-image."""

    fixtures = ['stacksimage.json']
    maxDiff = None

    def setUp(self):
        """Set up the test suite."""
        password = '12345'
        user = User.objects.create_user(
            username='test_user',
            email='user@test.com',
            password=password
        )
        user.is_active = True
        user.is_staff = True
        user.is_superuser = True
        user.save()
        user_client = Client()
        user_login = user_client.login(
            username='test_user',
            password=password
        )
        self.assertTrue(user_login)
        self.user = user
        self.user_client = user_client
        self.imagelist = StacksImageList.objects.get()

    def test_instances(self):
        """Test the test StacksImageList instance."""
        self.assertEqual(
            self.imagelist.pk,
            1
        )
        self.assertEqual(
            self.imagelist.__str__(),
            'Test Image List'
        )
        self.assertEqual(
            self.imagelist.images.all()[0].__str__(),
            'Django Logo'
        )
        self.assertEqual(
            self.imagelist.stacksimagelistimage_set.all()[
                0
            ].__str__(),
            'Test Image List 1. Django Logo'
        )

    def test_list_serialization(self):
        """Test the StacksImageList textplusstuff serializer."""
        response = self.client.get(
            '/textplusstuff/stacks_image/'
            'stacksimagelist/detail/1/'
        )
        self.assertEqual(
            response.status_code,
            200
        )
        self.assertEqual(
            json.loads(response.content)['context'],
            {
                "name": "Test Image List",
                "display_title": "A test list of images",
                "extra_context": {},
                "images": [
                    {
                        "name": "Django Logo",
                        "display_title": "The Django Logo",
                        "extra_context": {},
                        "image": {
                            "full_size": (
                                "/media/stacks_image/django-logo-green.png"
                            )
                        },
                        "optional_content": {
                            "as_plaintext": (
                                "This is a picture of the django logo!\n"
                            ),
                            "as_html": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "django logo</strong>!</p>\n"
                            ),
                            "raw_text": (
                                "This is a _picture_ of the **django logo**!"
                            ),
                            "as_html_no_tokens": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "django logo</strong>!</p>\n"
                            ),
                            "as_markdown": (
                                "This is a _picture_ of the **django logo**!"
                            )
                        }
                    },
                    {
                        "name": "Django Pony",
                        "display_title": "",
                        "extra_context": {},
                        "image": {
                            "full_size": (
                                "/media/stacks_image/django-pony-pink.png"
                            )
                        },
                        "optional_content": {
                            "as_plaintext": (
                                "This is a picture of the django pony.\n"
                            ),
                            "as_html": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "django pony</strong>.</p>\n"
                            ),
                            "raw_text": (
                                "This is a _picture_ of the **django pony**."
                            ),
                            "as_html_no_tokens": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "django pony</strong>.</p>\n"
                            ),
                            "as_markdown": (
                                "This is a _picture_ of the **django pony**."
                            )
                        }
                    },
                    {
                        "name": "Python Logo",
                        "display_title": "",
                        "extra_context": {},
                        "image": {
                            "full_size": "/media/stacks_image/python-logo.png"
                        },
                        "optional_content": {
                            "as_plaintext": (
                                "This is a picture of the python logo.\n"
                            ),
                            "as_html": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "python logo</strong>.</p>\n"
                            ),
                            "raw_text": (
                                "This is a _picture_ of the **python logo**."
                            ),
                            "as_html_no_tokens": (
                                "<p>This is a <em>picture</em> of the <strong>"
                                "python logo</strong>.</p>\n"
                            ),
                            "as_markdown": (
                                "This is a _picture_ of the **python logo**."
                            )
                        }
                    }
                ]
            }
        )



from streamfield_tools.registry import block_registry

from .blocks import (
    stacks_image_block,
    stacks_imagelist_block
)

block_registry.register_block(
    'image',
    stacks_image_block
)

block_registry.register_block(
    'image_list',
    stacks_imagelist_block
)



# -*- coding: utf-8 -*-
from distutils.core import setup
from setuptools import find_packages

setup(
    name='wagtail-stacks-image',
    packages=find_packages(),
    version='0.1',
    author=u'Jonathan Ellenberger',
    author_email='jonathan_ellenberger@wgbh.org',
    url='http://stacks.wgbhdigital.org/',
    license='MIT License, see LICENSE',
    description=(
        "A Wagtail Stacks application for handling images and lists of images."
    ),
    long_description=open('README.rst').read(),
    zip_safe=False,
    install_requires=['wagtail-streamfieldtools>=0.1'],
    include_package_data=True,
    package_data={
        'wagtail_stacks_image': [
            'static/sass/*.scss',
            'static/js/*.js',
            'static/js/vendor/lazyimages/*.js',
            'static/js/vendor/owl/*.js',
            'templates/wagtail_stacks_image/single/*.html',
            'templates/wagtail_stacks_image/list/*.html'
        ],
    },
    classifiers=[
        'Framework :: Django',
        'Framework :: Django :: 1.8',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7'
    ]
)
